#include "main.h"


#define BIT_TIME	104		// 9600 Baud

#define BIT_TIME_TX	(XTAL / BAUD)		// 9600 Baud

#ifdef RX_INVERT
#define get_rxd()	(RXD_PIN == 0)
#else
#define get_rxd()	(RXD_PIN == 1)
#endif

#ifdef TX_INVERT
#define set_txd(x)	(STXD_PORT = !x)
#else
#define set_txd(x)	(STXD_PORT = x)
#endif


u8 srx_tmp;
u8 srx_data;
u8 srx_mask;
volatile u8 srx_done;

u8 stx_state;
u8 stx_data;

void suart_rx_init( void )
{
  srx_done = 0;

  srx_mask = 0;				// also change direction flag
  //PCMSK = 1<<RXD_INT;			// enable start detection pin
  PCMSK2 = 1<<RXD_INT;
  //GIMSK = 1<<PCIE;
  PCICR = 1<<PCIE2;

  //TCCR1 = 0;
  TCCR0B = 0;
  GTCCR = 0;
  //OCR1A = BIT_TIME;
  OCR0A = BIT_TIME;
  //OCR1C = BIT_TIME;
  OCR0B = BIT_TIME;
  //TIMSK = 1<<OCIE1A;
  //TIMSK0 = 1<<OCIE0A;
}

void suart_tx_init( void )
{
  set_txd(1);					// inactive
  STXD_DDR = 1;					// output

  stx_state = 0;
  TCCR1A = 0;
  TCCR1B = 1<<CS10^1<<WGM12;			// clear on OCR1A

  OCR1A = BIT_TIME_TX;
  //TIMSK1 = 1<<OCIE1A;
}

u8 sgetchar( void )
{
  while( !srx_done );
  srx_done = 0;
  return srx_data;
}


//ISR( PCINT0_vect )
ISR( SIG_PIN_CHANGE2 )
{
  if( get_rxd() == 0 ){			// if start bit
    //TCNT1 = 256 - (BIT_TIME/2);         // 1.5 bit time until compare
	TCNT0 = 256 - (BIT_TIME/2);
    //TCCR1 = 1<<CTC1^1<<CS12;		// start T1 with XTAL / 8
	TCCR0B = 1<<CS01;
	TCCR0A = 1<<WGM01^1<<COM0A1;
    srx_tmp = 0;			// clear bit storage
    srx_mask = 1;			// start with LSB
    //PCMSK = 0;				// disable start detection
	PCMSK2 = 0;
  }
}


//ISR( TIM1_COMPA_vect )
ISR( SIG_OUTPUT_COMPARE0A )
{
  if( srx_mask ){
    if( get_rxd() == 1 )
      srx_tmp |= srx_mask;
    srx_mask <<= 1;
    return;
  }
  if( get_rxd() == 1 ){			// Stop bit valid
    srx_done = 1;			// mark rx data valid
    srx_data = srx_tmp;			// store rx data
    //TCCR1 = 0;				// stop T1
	TCCR0B = 0;
    //PCMSK = 1<<RXD_INT;			// enable further start detection
	PCMSK2 = 1<<RXD_INT;
  }
}

NIL(void sputchar( u8 c ))			// no inline in sputs
{
  while( vu8(stx_state) );			// volatile for test
  stx_data = c;
  stx_state = 10;
}


void sputs( u8 *s )
{
//PORTB = 0xFE;
  while( *s )
	//PORTB = 0xFE;
    sputchar( *s++ );
//PORTB = 0xFE;
}


//ISR( TIM1_COMPA_vect )
ISR( SIG_OUTPUT_COMPARE1A )
{
  switch( stx_state ){
    case 10: set_txd( 0 ); break;		// start bit
    case 9:
    case 8:
    case 7:
    case 6:
    case 5:
    case 4:
    case 3:
    case 2: if( !(stx_data & 1) ){
	      set_txd( 0 );			// data bit 0
	    }else{
    case 1:   set_txd( 1 );			// data bit 1 or stop
	    }
	    stx_data >>= 1;
	    break;
    case 0: return;				// done, do nothing
  }
  stx_state--;
}
